"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.validater = exports.getTemplateUrl = void 0;
const path_1 = __importDefault(require("path"));
const lodash_1 = __importDefault(require("lodash"));
const semver_1 = __importDefault(require("semver"));
const validate_npm_package_name_1 = __importDefault(require("validate-npm-package-name"));
const core_1 = require("../core");
exports.getTemplateUrl = async (input) => {
    if (/^https?:/.test(input))
        return input;
    input = input.includes('/') ? input : `zce-templates/${input}`;
    input = input.includes('#') ? input : `${input}#master`;
    const [owner, name, branch] = input.split(/\/|#/);
    const defaultRegistry = 'https://github.com/${owner}/${name}/archive/${branch}.zip';
    const { registry = defaultRegistry } = await core_1.config.get();
    return core_1.strings.render(registry, { owner, name, branch });
};
exports.validater = {
    name: input => {
        var _a;
        const result = validate_npm_package_name_1.default(input);
        if (result.validForNewPackages)
            return true;
        return ((_a = result.errors) === null || _a === void 0 ? void 0 : _a.join(', ')) || '';
    },
    version: input => {
        const valid = semver_1.default.valid(input);
        if (valid)
            return true;
        return `The '${input}' is not a semantic version.`;
    }
};
const generater = core_1.ware();
generater.use(async (context, next) => {
    context.dest = path_1.default.resolve(context.project);
    const exists = await core_1.file.exists(context.dest);
    if (!exists)
        return next();
    if (exists !== 'dir')
        throw new Error(`Cannot init ${context.project}: File exists.`);
    if (await core_1.file.isEmpty(context.dest))
        return next();
    core_1.logger.clear();
    const { sure } = await core_1.prompt({
        type: 'confirm',
        name: 'sure',
        initial: false,
        message: context.dest === process.cwd() ? 'Generate project in current directory?' : 'Target directory already exists. Continue?'
    });
    if (!sure)
        throw new Error('You have to cancel the init task.');
    const { choose } = await core_1.prompt({
        type: 'select',
        name: 'choose',
        message: 'Target directory is not empty. Pick an action:',
        choices: ['Merge', 'Overwrite', 'Cancel']
    });
    if (choose === 'Cancel')
        throw new Error('You have to cancel the init task.');
    if (choose === 'Overwrite') {
        await core_1.file.remove(context.dest);
    }
    return next();
});
generater.use(async (context, next) => {
    if (/^[./]|^[a-zA-Z]:/.test(context.template)) {
        context.src = path_1.default.resolve(context.template);
        return next();
    }
    context.url = await exports.getTemplateUrl(context.template);
    const hash = core_1.strings.md5(context.url);
    context.src = core_1.file.getDataPath('generator', hash);
    const exists = await core_1.file.isDirectory(context.src);
    if (context.offline) {
        if (exists) {
            core_1.logger.info(`🚆  Use cached template @ \`${core_1.logger.color.yellow(core_1.file.tildify(context.src))}\`.`);
            return next();
        }
        core_1.logger.info(`😔  Template cache \`${core_1.logger.color.yellow(core_1.file.tildify(context.src))}\` not found.`);
    }
    exists && await core_1.file.remove(context.src);
    const spinner = core_1.logger.spin('Downloading template...');
    try {
        spinner.start();
        const temp = await core_1.http.download(context.url);
        await core_1.file.extract(temp, context.src, 1);
        await core_1.file.remove(temp);
        spinner.succeed('Download template complete.');
        return next();
    }
    catch (e) {
        spinner.fail('Download failed.');
        throw new Error(`Failed to fetch template \`${context.template}\`: ${e.message}.`);
    }
});
generater.use(async (context, next) => {
    try {
        context.options = require(context.src);
        if (Object.prototype.toString.call(context.options) !== '[object Object]') {
            throw new TypeError('template needs to expose an object.');
        }
        return next();
    }
    catch (e) {
        if (e.code !== 'MODULE_NOT_FOUND') {
            e.message = `This template is invalid: ${e.message}`;
            throw e;
        }
    }
    context.options = { name: context.template };
    return next();
});
generater.use(async (context, next) => {
    var _a;
    if (!((_a = context.options) === null || _a === void 0 ? void 0 : _a.plugin))
        return next();
    return context.options.plugin(context, next);
});
generater.use(async (context, next) => {
    if (!context.options)
        return next();
    if (!context.options.questions) {
        context.options.questions = { name: 'name', type: 'input', message: 'Project name' };
    }
    if (!Array.isArray(context.options.questions)) {
        context.options.questions = [context.options.questions];
    }
    const npmrc = await core_1.config.npm();
    const yarnrc = await core_1.config.yarn();
    const gitconfig = await core_1.config.git();
    await Promise.all(context.options.questions.map(async (item) => {
        switch (item.name) {
            case 'name':
                item.initial = item.initial || path_1.default.basename(context.dest);
                item.validate = item.validate || exports.validater.name;
                break;
            case 'version':
                item.validate = exports.validater.version;
                item.initial = item.initial || (npmrc === null || npmrc === void 0 ? void 0 : npmrc['init-version']) || (yarnrc === null || yarnrc === void 0 ? void 0 : yarnrc['init-version']) || '0.1.0';
                break;
            case 'author':
                item.initial = item.initial || (npmrc === null || npmrc === void 0 ? void 0 : npmrc['init-author-name']) || (yarnrc === null || yarnrc === void 0 ? void 0 : yarnrc['init-author-name']) || (gitconfig === null || gitconfig === void 0 ? void 0 : gitconfig['user.name']);
                break;
            case 'email':
                item.initial = item.initial || (npmrc === null || npmrc === void 0 ? void 0 : npmrc['init-author-email']) || (yarnrc === null || yarnrc === void 0 ? void 0 : yarnrc['init-author-email']) || (gitconfig === null || gitconfig === void 0 ? void 0 : gitconfig['user.email']);
                break;
            case 'url':
                item.initial = item.initial || (npmrc === null || npmrc === void 0 ? void 0 : npmrc['init-author-url']) || (yarnrc === null || yarnrc === void 0 ? void 0 : yarnrc['init-author-url']) || (gitconfig === null || gitconfig === void 0 ? void 0 : gitconfig['user.url']);
                break;
            case 'license':
                item.initial = item.initial || (npmrc === null || npmrc === void 0 ? void 0 : npmrc['init-license']) || (yarnrc === null || yarnrc === void 0 ? void 0 : yarnrc['init-license']) || 'MIT';
                break;
        }
    }));
    context.answers = await core_1.prompt(context.options.questions);
    return next();
});
generater.use(async (context, next) => {
    var _a, _b;
    const cwd = path_1.default.join(context.src, ((_a = context.options) === null || _a === void 0 ? void 0 : _a.source) || 'template');
    let filenames = await core_1.file.glob('**', { dot: true, nodir: true, cwd });
    const filters = (_b = context.options) === null || _b === void 0 ? void 0 : _b.filters;
    if (filters) {
        const patterns = Object.keys(filters).filter(item => !filters[item](context.answers));
        const mmOptions = { dot: true, matchBase: true };
        patterns.forEach(pattern => {
            filenames = filenames.filter(filename => !core_1.file.minimatch(filename, pattern, mmOptions));
        });
    }
    const files = {};
    await Promise.all(filenames.map(item => {
        return core_1.file.read(path_1.default.join(cwd, item)).then(buffer => { files[item] = buffer; });
    }));
    context.files = files;
    await next();
});
generater.use(async (context, next) => {
    const files = context.files;
    Object.keys(files).forEach(original => {
        const current = core_1.strings.render(original, context.answers);
        if (current === original)
            return;
        files[current] = files[original];
        delete files[original];
    });
    await next();
});
generater.use(async (context, next) => {
    const files = context.files;
    Object.keys(files).forEach(name => {
        var _a, _b;
        const contents = files[name].toString();
        if (!(/<%([\s\S]+?)%>/.test(contents) || /\$\{([^\\}]*(?:\\.[^\\}]*)*)\}/.test(contents))) {
            return;
        }
        files[name] = Buffer.from(core_1.strings.render(contents, context.answers, {
            imports: { _: lodash_1.default, ...(_a = context.options) === null || _a === void 0 ? void 0 : _a.metadata, ...(_b = context.options) === null || _b === void 0 ? void 0 : _b.helpers }
        }));
    });
    await next();
});
generater.use(async (context, next) => {
    const dest = context.dest;
    const files = context.files;
    await Promise.all(Object.keys(files).map(name => core_1.file.write(path_1.default.join(dest, name), files[name])));
    await next();
});
generater.use(async (context, next) => {
    if (Object.keys(context.files).includes('package.json')) {
        const useYarn = Object.keys(context.files).includes('yarn.lock');
        const spinner = core_1.logger.spin('Installing dependencies...');
        try {
            await core_1.system.exec(useYarn ? 'yarn' : 'npm', ['install'], { cwd: context.dest });
            spinner.succeed('Install deps completed.');
        }
        catch (e) {
            spinner.fail('Install deps failed: ' + e.message);
        }
    }
    await next();
});
generater.use(async (context, next) => {
    const spinner = core_1.logger.spin('Initializing repository...');
    try {
        await core_1.system.exec('git', ['init'], { cwd: context.dest });
        await core_1.system.exec('git', ['add', '--all'], { cwd: context.dest });
        await core_1.system.exec('git', ['commit', '-m', 'feat: initial commit'], { cwd: context.dest });
        spinner.succeed('Initial repo completed.');
    }
    catch (e) {
        spinner.fail('Initial repo failed: ' + e.message);
    }
    await next();
});
generater.use(async (context, next) => {
    var _a;
    if ((_a = context.options) === null || _a === void 0 ? void 0 : _a.complete) {
        return context.options.complete(context, next);
    }
});
const command = {
    name: 'init',
    usage: 'init <template> [project]',
    description: 'generate a new project from a template.',
    options: {
        offline: {
            type: 'boolean',
            alias: 'o',
            default: false,
            description: 'offline mode, use cached template'
        }
    },
    examples: [
        core_1.logger.color.gray('# create a new project with an official template'),
        '$ [bin] init <template> [project]',
        core_1.logger.color.gray('# create a new project straight from a github template'),
        '$ [bin] init <owner>/<repo> [project]'
    ],
    action: async ({ primary: template, secondary: project = '.', options }) => {
        const { offline } = options;
        if (!template) {
            return core_1.missingArgument('template');
        }
        try {
            await generater.run({ template, project, offline });
        }
        catch (e) {
            core_1.logger.error(e);
        }
    }
};
exports.default = command;
